///////////////////////////////////////////////////////////////////////////////////////////////
//
//  This sample source code comes with Servosila SC-25C Brushless Motor Controllers.
//
//  The code is provided "AS IS" without any kind of guarantees or warranties.
//  Use it at your own risk.
//
//  The code is free for everyone to use, modify or redistribute.
//
//  www.servosila.com
//
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SERVOSILA_CANOPEN_DECODER_H
#define SERVOSILA_CANOPEN_DECODER_H

#include <stddef.h> //size_t
#include <stdint.h>
#include <math.h>

namespace servosila
{

//Extract Node ID from CAN ID
inline uint32_t extract_node_id_from_can_id(uint32_t can_id)
{
    //assert(can_id<=2047); //11bit only
    const uint32_t node_id = can_id & 127; //0000-1111111
    return node_id;
}

//Extract COB ID from CAN ID
inline uint32_t extract_cob_id_from_can_id(uint32_t can_id)
{
    //assert(can_id<=2047); //11bit only
    const uint32_t cob_id = can_id & 0x780; //1111-0000000
    return cob_id;
}

#define MAX_FLOAT16     ( 128.0f)
#define MIN_FLOAT16     (-MAX_FLOAT16)

inline int16_t encode_float16(float f32)
{
        //clipping the value
        if(f32 > MAX_FLOAT16) f32 = MAX_FLOAT16;
        if(f32 < MIN_FLOAT16) f32 = MIN_FLOAT16;
        //scaling
        const int16_t bits = lroundf(f32 * (32767.0f / MAX_FLOAT16));
        return bits;
}

inline float decode_float16(int16_t bits)
{
        //de-scaling
        const float f32 = float(bits) * (MAX_FLOAT16 / 32767.0f);
        return f32;
}
} //namespace

#endif
