///////////////////////////////////////////////////////////////////////////////////////////////
//
//  This sample source code comes with Servosila SC-25C Brushless Motor Controllers.
//
//  The code is provided "AS IS" without any kind of guarantees or warranties.
//  Use it at your own risk.
//
//  The code is free for everyone to use, modify or redistribute.
//
//  www.servosila.com
//
///////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SERVOSILA_SLCAN_ENCODER_H
#define SERVOSILA_SLCAN_ENCODER_H

#include <stdint.h>
#include <stddef.h> //size_t

namespace servosila
{

//Type definitions
//CAN payload - just an array of 8 bytes
typedef uint8_t payload_t[8];
//a buffer for a SLCAN text message
typedef char slcan_message_t[27];     //Maximum message size is of a 29bits ID message: T 1FFFFFFF 8 0123456789ABCDEF[CR]

//converts a integer in range 0..15 into a corresponding hex symbol
inline uint8_t int2hex(uint8_t n)
{
    uint8_t result;

    if(n<10) result = '0' + n;
    else     result = 'A' + n-10;

    return result;
}

//Encoding an SLCAN text message
//...the inputs are Node ID, COB ID, and Payload.
//...the output is an SLCAN text message
//...returns the number of chars in SLCAN message
//Note that the result is NOT a zero-terminated string, but just an array of chars.
inline size_t slcan_encode_11bit(uint32_t can_id, const payload_t& payload, uint16_t payload_length, slcan_message_t& message)
{
    message[0] = 't';

    //CAN-ID
    message[1] = int2hex((can_id & (0xFul << 2ul*4ul)) >> 2ul*4ul);
    message[2] = int2hex((can_id & (0xFul << 1ul*4ul)) >> 1ul*4ul);
    message[3] = int2hex( can_id &  0xFul                        );

    //Length of payload
    message[4] = int2hex(payload_length);

    //Payload encoding: LoWord
    for(size_t i=0; (i<8)&&(i<payload_length); i++)
    {
        message[ 5 + 2*i  ] = int2hex((payload[i] & 0xF0ul) >> 4ul);
        message[ 5 + 2*i+1] = int2hex(payload[i] & 0x0Ful);
    }

    //Adding [CR]
    message[1 + 3 + 1 + payload_length*2] =  13;

    //Computing the total size of the string
    //             t  ID   L   PAYLOAD        CR
    const size_t message_size = 1 + 3 + 1 + payload_length*2 + 1;

    return message_size;
}

} //namespace servosila

#endif
